Program test

#include "petsc/finclude/petscdmda.h"
  use petscdmda

  DM       :: da
  Mat      :: preA, A
  Vec      :: x
  PetscInt :: nx, ny, nz, ngrow3, mloc3
  PetscErrorCode :: ierr

  ! Mapping to use mat stencil routines
  ISLocalToGlobalMapping :: ltog
  PetscInt  :: gstart(0:2), gdim(0:2)

  PetscInt, parameter :: i3=3


  call PetscInitialize(PETSC_NULL_CHARACTER,ierr)

  ! Create DA with dof=3 to handle 3 component vectors
  nx=100
  ny=100
  nz=100
  call DMDACreate3d(PETSC_COMM_WORLD,DM_BOUNDARY_NONE,DM_BOUNDARY_NONE,DM_BOUNDARY_NONE, &
    DMDA_STENCIL_BOX,nx,ny,nz,PETSC_DECIDE,PETSC_DECIDE,PETSC_DECIDE,i3,i3,PETSC_NULL_INTEGER, &
    PETSC_NULL_INTEGER,PETSC_NULL_INTEGER, da,ierr)
  call DMSetUp(da,ierr)

  ! Get local to global mapping for this DM
  call DMGetLocalToGlobalMapping(da,ltog,ierr)
  call DMDAGetGhostCorners(da,gstart(0),gstart(1),gstart(2),gdim(0),gdim(1),gdim(2),ierr)

  ! Get local size
  call DMGetGlobalVector(da,x,ierr)
  call VecGetLocalSize(x,mloc3,ierr)
  call DMRestoreGlobalVector(da,x,ierr)

  ! Use PETSc preallocator matrix to determine preallocation
  ngrow3=nx*ny*nz*3
  call MatCreate(PETSC_COMM_WORLD,preA,ierr)
  call MatSetSizes(preA,mloc3,mloc3,ngrow3,ngrow3,ierr)
  call MatSetType(preA,MATPREALLOCATOR,ierr)
  call MatSetLocalToGlobalMapping(preA,ltog,ltog,ierr)
  call MatSetStencil(preA,i3,gdim,gstart,i3,ierr)
  call MatSetUp(preA,ierr)

  ! Go through subroutine to set coefficient values
  call setA(da,preA,nx,ny,nz)

  ! Assemble the preallocator matrix
  call MatAssemblyBegin(preA,MAT_FINAL_ASSEMBLY,ierr)
  call MatAssemblyEnd(preA,MAT_FINAL_ASSEMBLY,ierr)

  ! Create the coefficient matrix
  call MatCreate(PETSC_COMM_WORLD,A,ierr)
  call MatSetSizes(A,mloc3,mloc3,ngrow3,ngrow3,ierr)
  call MatSetType(A,MATAIJ,ierr)
  call MatPreallocatorPreallocate(preA,.false.,A,ierr)
  call MatSetLocalToGlobalMapping(A,ltog,ltog,ierr)
  call MatSetStencil(A,i3,gdim,gstart,i3,ierr)

  ! Go through subroutine to set coefficient values
  call setA(da,A,nx,ny,nz)

  ! Assemble the preallocator matrix
  call MatAssemblyBegin(A,MAT_FINAL_ASSEMBLY,ierr)
  call MatAssemblyEnd(A,MAT_FINAL_ASSEMBLY,ierr)

  ! Release space
  call DMDestroy(da,ierr)
  call MatDestroy(preA,ierr)
  call MatDestroy(A,ierr)

  call PetscFinalize(ierr)


end program test


!==========================================================================================
subroutine setA(da,A,nx,ny,nz)

#include "petsc/finclude/petscksp.h"
  use petscksp


  DM       :: da
  Mat      :: A
  MatStencil :: row(4), col(4,3)
  PetscScalar :: v(1)
  PetscInt  :: xm, ym, zm, xs, ys, zs, xe, ye, ze
  PetscInt  :: i, j, k, ii, jj, kk, iComp
  PetscInt  :: nx, ny, nz
  PetscErrorCode :: ierr

  PetscInt, parameter :: i1=1


  ! Get the grid info for each processor
  call DMDAGetCorners(da,xs,ys,zs,xm,ym,zm,ierr)

  ! Shift the grid coordinates by 1 so we handle the Fortran convention
  ! of 1-based indices rather than 0-based indices.
  xs = xs + 1
  ys = ys + 1
  zs = zs + 1
  xe = xs + xm - 1
  ye = ys + ym - 1
  ze = zs + zm - 1

  ! Loop over the local entries of the process
  do k=zs,ze
    do j=ys,ye
      do i=xs,xe
        do iComp=0,2

          ! We need 0 based indexing, but we shifted xsG, ysG, etc by 1 in the main program, so we shift back here
          ii=i-1
          jj=j-1
          kk=k-1

          ! Set row
          row(MatStencil_i)=ii
          row(MatStencil_j)=jj
          row(MatStencil_k)=kk
          row(MatStencil_c)=iComp

          ic=1
          v(ic) = 2.d0
          col(MatStencil_i,ic)=ii
          col(MatStencil_j,ic)=jj
          col(MatStencil_k,ic)=kk
          col(MatStencil_c,ic)=iComp

          if (i /= 1) then
            ic=ic+1
            v(ic) = -1.d0
            col(MatStencil_i,ic)=ii-1
            col(MatStencil_j,ic)=jj
            col(MatStencil_k,ic)=kk
            col(MatStencil_c,ic)=iComp
          end if

          if (i /= nx) then
            ic=ic+1
            v(ic) = -1.d0
            col(MatStencil_i,ic)=ii+1
            col(MatStencil_j,ic)=jj
            col(MatStencil_k,ic)=kk
            col(MatStencil_c,ic)=iComp
          end if

        end do

        ! Set this row of the matrix
        call MatSetValuesStencil(A,i1,row,ic,col,v,ADD_VALUES,ierr)

      end do
    end do
  end do


end subroutine setA

